package com.accmee.quiz.application;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import oracle.adfmf.application.LifeCycleListener;
import oracle.adfmf.framework.api.AdfmfJavaUtilities;


/**
 * This auto-generated application life cycle listener provides the basic structure for
 * developers needing to include their own functionality during the different stages of
 * the application life cycle.
 *
 * Common examples of functionality that might be added:
 *
 * start:
 *   1. determine if the application has updates
 *   2. determine if there already exists a local application database image
 *   3. setup any one time state for the application
 *
 * activate:
 *   1. read any application cache stores and re-populate state (if needed)
 *   2. establish/re-establish any database connections and cursors
 *   3. process any pending web-service requests
 *   4. obtain any required resources
 *
 * deactivate:
 *   1. write any restorable state to an application cache store
 *   2. close any database cursors and connections
 *   3. defer any pending web-service requests
 *   4. release held resources
 *
 * stop:
 *   1. logoff any remote services
 *
 * @see oracle.adfmf.application.LifeCycleListener
 *
 * NOTE:
 * 1. Even though this is auto generated, it is not auto registered (see #2) since doing
 *    so could incur startup of the JVM in cases where it may not otherwise be necessary.
 * 2. In order for the system to recognize an application life cycle listener
 *    it must be registered in the adfmf-application.xml file.
 * 3. Application assemblers must implement this interface if they would like to
 *    receive notification of application start, hibernation, and application resume.
 */
public class LifeCycleListenerImpl implements LifeCycleListener
{
    public LifeCycleListenerImpl()
    {
    }

    /**
     * The start method will be called at the start of the application.
     *
     * NOTE:
     * 1. This is a <b>blocking</b> call and will freeze the user interface
     *    while the method is being executed.  If you have any longer running
     *    items you should create a background thread and do the work there.
     * 2. Only the application controller's classes will be available in this
     *    method.
     */
    public void start()
    {
        try
        {
            Statement stat = DBConnectionFactory.getConnection().createStatement();
            ResultSet rs = stat.executeQuery("SELECT * FROM wordcouples;");
            if (!rs.next())
                InitDB();
        }
        catch (SQLException e)
        {
            // if the error message is "out of memory",
            // it probably means no database file is found
            InitDB();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * The stop method will be called at the termination of the application.
     *
     * NOTE:
     * 1. Depending on how the application is being shutdown, this method may
     *    or may not be called. For example, if a user kills the Application from
     *    the iOS multitask UI then stop will not be called.  Because of this, each
     *    feature should save off their state in the deactivate handler.
     * 2. Only the application controller's classes will be available in this
     *    method.
     */
    public void stop()
    {
        // Add code here...
    }

    /**
     * The activate method will be called when the application is started (post
     * the start method) and when an application is resumed by the operating
     * system.
     *
     * NOTE:
     * 1. This is a <b>blocking</b> call and will freeze the user interface
     *    while the method is being executed.  If you have any longer running
     *    items you should create a background thread and do the work there.
     * 2. Only the application controller's classes will be available in this
     *    method.
     */
    public void activate()
    {
        // Add code here...
    }

    /**
     * The deactivate method will be called as part of the application shutdown
     * process or when the application is being deactivated/hibernated by the
     * operating system.
     *
     * NOTE:
     * 1. This is a <b>blocking</b> call and will freeze the user interface
     *    while the method is being executed.  If you have any longer running
     *    items you should create a background thread and do the work there.
     * 2. Only the application controller's classes will be available in this
     *    method.
     */
    public void deactivate()
    {
        // Add code here...
    }

    private void InitDB()
    {
        try
        {
            copyDataBase();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        //        try
        //        {
        //            ClassLoader cl = Thread.currentThread().getContextClassLoader();
        //            InputStream is = cl.getResourceAsStream(".adf/META-INF/locations.sql");
        //            if (is == null)
        //            {
        //                System.err.println("###### Could not look up : /META-INF/locations.sql");
        //                return;
        //            }
        //
        //            BufferedReader bReader = new BufferedReader(new InputStreamReader(is));
        //            List stmts = new ArrayList();
        //            String strstmt = "";
        //            String ln = bReader.readLine();
        //            while (ln != null)
        //            {
        //                if (ln.startsWith("REM") || ln.startsWith("COMMIT"))
        //                {
        //                    ln = bReader.readLine();
        //                    continue;
        //                }
        //                strstmt = strstmt + ln;
        //                if (strstmt.endsWith(";"))
        //                {
        //                    stmts.add(strstmt);
        //                    strstmt = "";
        //                    ln = bReader.readLine();
        //                    continue;
        //                }
        //                ln = bReader.readLine();
        //            }
        //
        //            DBConnectionFactory.getConnection().setAutoCommit(false);
        //            for (int i = 0; i < stmts.size(); i++)
        //            {
        //                Statement pStmt = DBConnectionFactory.getConnection().createStatement();
        //                pStmt.executeUpdate((String)stmts.get(i));
        //
        //            }
        //            DBConnectionFactory.getConnection().commit();
        //        }
        //        catch (Exception e)
        //        {
        //            e.printStackTrace();
        //        }

    }

    /**
     * Copies your database from your local assets-folder to the just created empty database in the
     * system folder, from where it can be accessed and handled.
     * This is done by transfering bytestream.
     * */
    private void copyDataBase() throws IOException
    {
        //Open your local db as the input stream
        ClassLoader cl = Thread.currentThread().getContextClassLoader();

        // Path to the just created empty db
        String inputFileName = ".adf/META-INF/dictionary.db";
        InputStream myInput = cl.getResourceAsStream(inputFileName);
        if (myInput == null)
        {
            System.err.println("###### Could not look up : " + inputFileName);
            return;
        }
        String outFileName = AdfmfJavaUtilities.getDirectoryPathRoot(AdfmfJavaUtilities.ApplicationDirectory)+"/dictionary.db";
        //Open the empty db as the output stream
        OutputStream myOutput = new FileOutputStream(outFileName);

        //transfer bytes from the inputfile to the outputfile
        byte[] buffer = new byte[1024];
        int length;
        while ((length = myInput.read(buffer)) > 0)
        {
            myOutput.write(buffer, 0, length);
        }

        //Close the streams
        myOutput.flush();
        myOutput.close();
        myInput.close();

    }
}
