package com.accmee.quiz.mobile;

import oracle.adfmf.java.beans.PropertyChangeListener;
import oracle.adfmf.java.beans.PropertyChangeSupport;

public class QuizDataObject
{
    private int id;
    private String word;
    private String interpretedWord;
    String options[] = new String[3];
    private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    public QuizDataObject(int id, String word, String interpretedWord)
    {
        super();
        this.id = id;
        this.word = word;
        this.interpretedWord = interpretedWord;
    }

    public QuizDataObject()
    {
        super();
    }

    public void setOption(int index, String option)
    {
        options[index] = option;
    }

    public String getOption(int index)
    {
        return options[index];
    }

    public boolean isOptionFilled(int index)
    {
        return getOption(index) == null ? false : getOption(index).length() > 0;
    }

    public boolean isRight(String option)
    {
        return option.equals(getInterpretedWord());
    }

    public String toString()
    {
        return "id: " + getId() + " - word: " + getWord() + " - interpretedword: " + getInterpretedWord();
    }

    public void setId(int id)
    {
        int oldId = this.id;
        this.id = id;
        propertyChangeSupport.firePropertyChange("id", oldId, id);
    }

    public void addPropertyChangeListener(PropertyChangeListener l)
    {
        propertyChangeSupport.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l)
    {
        propertyChangeSupport.removePropertyChangeListener(l);
    }

    public int getId()
    {
        return id;
    }

    public void setWord(String word)
    {
        String oldWord = this.word;
        this.word = word;
        propertyChangeSupport.firePropertyChange("word", oldWord, word);
    }

    public String getWord()
    {
        return word;
    }

    public void setInterpretedWord(String interpretedWord)
    {
        String oldInterpretedWord = this.interpretedWord;
        this.interpretedWord = interpretedWord;
        propertyChangeSupport.firePropertyChange("interpretedWord", oldInterpretedWord, interpretedWord);
    }

    public String getInterpretedWord()
    {
        return interpretedWord;
    }

    public void setOptions(String[] options)
    {
        String[] oldOptions = this.options;
        this.options = options;
        propertyChangeSupport.firePropertyChange("options", oldOptions, options);
    }

    public String[] getOptions()
    {
        return options;
    }
}
