package com.accmee.quiz.mobile;

public class RandomNumber
{

    /**
     * This static function returns a java-generated random number between the values of 0 and 1.
     * @author Sun Microsystems
     */
    public static double GetJavaGeneratedRandomNumber()
    {
        // Return a Java-generated random number for the sake of completeness in this class.
        return Math.random();
    }

    /**
     * This function returns a random integer between two numbers, which are accepted as input. This random number generator is built off the output of Java's random number generator, except this one produces output between two values that are passed to the function.
     * @author Garrett Padera
     * @param low - The lowest random number you would like returned. (must be positive)
     * @param high - The highest random number you would like returned. (must be positive)
     * @return A random integer, in between or equal to the low and high values
     * @example GetRandomNumber(0, 10) will return a random number between 0 - 10.
     * @example GetRandomNumber(2, 200) will return a random number between 2 - 200.
     */
    public static int GetRandomNumber(int low, int high)
    {
        // Increment one to the high value, because otherwise the high value wouldn't be considered and therefore never returned.
        high++;

        // Determine how many possibilities we could return.
        int numPossibilities = (high - low);

        // We will be dividing by numPossibilities, so we cannot have it be 0!
        if (numPossibilities == 0)
            return low;

        // We will be getting a random number from Java that will be in between 0.0 and 1.0. The way this generator works is it takes the result returned from the Java random number generator, and depending on the number, assigns it to an interger between the high and low values.
        double incrementAmount = (1.0 / numPossibilities);

        // Fetch the random number.
        double randomNumJavaGenerated = Math.random();

        // Our system is not equipped to handle the rare event that we do get a perfect 1.0 as a random number from Java. If that is the case, just return the highest number instaed of watching the system malfunction.
        // But don't forget to deincrement the high number, which we incremented earlier for use with the system! Now that the system is being bypassed we do not need it incremented.
        if (randomNumJavaGenerated == 1.0)
            return --high;

        // Iterate through the list of possibilities. Take the range of possible returned numbers, and apply that to the range of numbers that we were given by java. (i.e. for generating a random number 0 - 10 our incrementAmount would be .1)
        int wholeNumber = low;
        for (float i = 0; wholeNumber <= high; i += incrementAmount)
        {
            if (randomNumJavaGenerated >= i && randomNumJavaGenerated < (i + incrementAmount))
                return wholeNumber;
            wholeNumber++;
        }

        // If for whatever reason we get through the loop without finding a number, return the higheset possible returnable number.
        return high;
    }

}
