package com.acme.structure;


import com.acme.structure.util.ObjectUtil;

import java.io.Serializable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Vector;


public class GridData implements Serializable
{
    private Collection synchBuffer;
    private Vector dataCollection;
    private Collection newBuffer, updatedBuffer, garbageData;
    private Collection selectedRowCollection;
    private Object dataObject;

    private int gridType;

    public final static int DATATYPE_MULTIROW = 0;
    public final static int DATATYPE_SINGLEROW = 1;

    public GridData()
    {
        super();
        this.gridType = DATATYPE_MULTIROW;
        initializeBuffers();
    }


    public GridData(int gridType)
    {
        super();
        this.gridType = gridType;
        initializeBuffers();
    }

    private void initializeBuffers()
    {
        dataCollection = new Vector();
        newBuffer = new ArrayList();
        updatedBuffer = new ArrayList();
        synchBuffer = new ArrayList();
        garbageData = new ArrayList();
        selectedRowCollection = new ArrayList();
    }

    public void clear()
    {
        dataObject = null;
        dataCollection.clear();
        newBuffer.clear();
        synchBuffer.clear();
        updatedBuffer.clear();
        garbageData.clear();
    }

    public void initializeGridData(Collection dataObjects)
    {
        clear();
        dataCollection.addAll(dataObjects);
        synchBuffer.addAll(dataObjects);
    }

    public void initializeGridData(Object baseEntity)
    {
        clear();
        dataObject = baseEntity;
        dataCollection.add(baseEntity);
        synchBuffer.add(baseEntity);
    }

    public void setDataCollection(Vector dataCollection)
    {
        this.dataCollection = dataCollection;
    }

    public void removeData(Object dataObject)
    {
        if (newBuffer.contains(dataObject))
            newBuffer.remove(dataObject);
        if (updatedBuffer.contains(dataObject))
        {
            updatedBuffer.remove(dataObject);
            garbageData.add(dataObject);
        } else
            garbageData.add(dataObject);
    }

    public void updateData(Object dataObject)
    {
        int index;
        if (newBuffer.contains(dataObject))
        {
            newBuffer.remove(dataObject);
            newBuffer.add(dataObject);
            index = dataCollection.indexOf(dataObject);
            dataCollection.remove(index);
            dataCollection.add(index, dataObject);
        } else if (updatedBuffer.contains(dataObject))
        {
            updatedBuffer.remove(dataObject);
            updatedBuffer.add(dataObject);
            index = dataCollection.indexOf(dataObject);
            dataCollection.remove(index);
            dataCollection.add(index, dataObject);
        } else if (synchBuffer.contains(dataObject))
        {
            //synchBuffer.remove(dataObject); //maybe required
            updatedBuffer.add(dataObject);
            index = dataCollection.indexOf(dataObject);
            dataCollection.remove(index);
            dataCollection.add(index, dataObject);
        }

        if (getGridType() == GridData.DATATYPE_SINGLEROW)
            this.dataObject = dataObject;
    }

    public void addUpdatedGridData(Object dataObject)
    {
        if (!newBuffer.contains(dataObject))
        {
            if (updatedBuffer.contains(dataObject))
                updatedBuffer.remove(dataObject);
            updatedBuffer.add(dataObject);
        }
    }

    public void addNewGridData(Object dataObject)
    {
        newBuffer.add(dataObject);
        dataCollection.add(dataObject);
    }

    public Vector getDataCollection()
    {
        return dataCollection;
    }

    public void setGarbageData(Collection garbageData)
    {
        this.garbageData = garbageData;
    }

    public Collection getGarbageData()
    {
        return garbageData;
    }

    public void setNewBuffer(Collection newBuffer)
    {
        this.newBuffer = newBuffer;
    }

    public Collection getNewBuffer()
    {
        return newBuffer;
    }

    public void setUpdatedBuffer(Collection updatedBuffer)
    {
        this.updatedBuffer = updatedBuffer;
    }

    public Collection getUpdatedBuffer()
    {
        return updatedBuffer;
    }

    public void setSynchBuffer(Collection synchBuffer)
    {
        this.synchBuffer = synchBuffer;
    }

    public Collection getSynchBuffer()
    {
        return synchBuffer;
    }

    public boolean isDataObjectUpdated()
    {
        if (updatedBuffer.contains(getDataObject()))
            return true;
        if (newBuffer.contains(getDataObject()))
            return true;

        return false;
    }

    public void setDataObject(Object dataObject)
    {
        this.dataObject = dataObject;
    }

    public Object getDataObject()
    {
        return dataObject;
    }

    public int getSize()
    {
        return dataCollection.size();
    }

    public void updateDataCollectionWithSynch(Collection synchBuffer)
    {
        synchBuffer.removeAll(getGarbageData());
        initializeGridData(synchBuffer);
    }

    public void updateObjectDataWithSynch(Collection synchBuffer)
    {
        synchBuffer.removeAll(getGarbageData());
        initializeGridData(ObjectUtil.getObject(synchBuffer, 0));
    }

    public void setSelectedRowCollection(Collection selectedRowCollection)
    {
        this.selectedRowCollection = selectedRowCollection;
    }

    public Collection getSelectedRowCollection()
    {
        return selectedRowCollection;
    }

    public void setGridType(int gridType)
    {
        this.gridType = gridType;
    }

    public int getGridType()
    {
        return gridType;
    }
}
