package com.acme.structure;


import com.acme.structure.util.ObjectUtil;

import java.io.Serializable;


public abstract class SuperEntity implements Serializable, Comparable
{
    protected int id;
    protected String entityName = "";

    public String getEntityName()
    {
        return entityName;
    }

    abstract public int getId();

    abstract public void setId(int id);

    public void setEntityName(String entityName)
    {
        this.entityName = entityName;
    }

    public boolean isPersisted()
    {
        return getId() == 0 ? false : true;
    }

    public boolean equals(Object object)
    {
        if (!(object instanceof SuperEntity))
            return false;

        SuperEntity tempEntity = (SuperEntity)object;
        if (ObjectUtil.isNull(tempEntity))
            return false;

        if (tempEntity.getEntityName().equals(getEntityName()))
        {
            if (tempEntity.isPersisted())
                return true;
            else
            {
                try
                {
                    return this.toString().equals(tempEntity.toString());
                }
                catch (Exception exc)
                {
                    return super.equals(tempEntity);
                }
            }
        }

        return super.equals(tempEntity);
    }

    public int hashCode()
    {
        if (isPersisted())
            return getId() + (getEntityName().hashCode() * 100);
        return super.hashCode();
    }

    public SuperEntity shallowCopy() throws CloneNotSupportedException
    {
        return (SuperEntity)super.clone();
    }

    public void clearIdFields()
    {
        setId(0);
    }

    abstract public String toString();

    public int compareTo(Object object)
    {
        if (!(object instanceof SuperEntity))
            throw new ClassCastException();

        SuperEntity superEntity = (SuperEntity)object;
        Integer idA = new Integer(getId());
        Integer idB = new Integer(superEntity.getId());

        return idA.compareTo(idB);
    }
}
